"""
xxydb — A股轻量数据库封装

基于 Parquet (Hive分区) + DuckDB，提供简洁的写入/查询 API。
"""

import json
import os
import re
import shutil
from pathlib import Path
from typing import List, Optional

import duckdb
import pandas as pd


class xxydb:
    """
    轻量存储管理。

    usage:
        db = xxydb(path='duckdb测试')
        db.write_data(data, id='cn_stock_bar1d', date_col='date', partitioning='年')
        result = db.query("SELECT * FROM cn_stock_bar1d WHERE date > '2024-01-01'").df()
    """

    _PART_MAP = {"年": "year", "月": "month", "日": "day"}

    def __init__(
        self,
        path: Optional[str] = None,
        api_key: Optional[str] = None,
        base_url: Optional[str] = None,
        model: Optional[str] = None,
    ):
        self.base_dir = Path(path) if path else Path.cwd()
        self.base_dir.mkdir(parents=True, exist_ok=True)

        self._config_path = self.base_dir / "tables_config.json"
        self._db_path = self.base_dir / "stock_database.duckdb"
        self._config = self._load_config()
        self._con = duckdb.connect(str(self._db_path))
        self._api_key = api_key
        self._base_url = base_url
        self._model = model

        # 启动时为已有表创建视图
        self._refresh_all_views()

    # ──────────────────────────────────────────
    # 上下文管理器
    # ──────────────────────────────────────────

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()
        return False

    def __repr__(self):
        return f"xxydb(path={str(self.base_dir)!r}, tables={len(self._config)})"

    # ──────────────────────────────────────────
    # 公开 API
    # ──────────────────────────────────────────

    def write_data(
        self,
        data: pd.DataFrame,
        id: str,
        date_col: str = "date",
        partitioning: Optional[str] = "年",
        unique_together: Optional[List[str]] = None,
        rewrite: bool = True,
        schema: Optional[dict] = None,
    ):
        """
        将 DataFrame 写入存储，支持历史批量和增量更新。

        参数:
            data:             要写入的 DataFrame
            id:               表名 / 存储 ID
            date_col:         日期列名，默认 "date"
            partitioning:     分区粒度，"年" / "月" / "日" / None(不分区)
            unique_together:  主键列表，指定后自动去重；None 则不去重
            rewrite:          True = 全量覆盖(删除旧数据后写入新数据)；False = 增量合并(有冲突时保留新值)
            schema:           字段描述，如 {"close": {"desc": "收盘价", "type": "float"}}
                              未提供时自动从 DataFrame 推断类型
        """
        if date_col not in data.columns:
            raise ValueError(f"日期列 '{date_col}' 不存在于 DataFrame 中")

        if partitioning is not None:
            part_key = self._PART_MAP.get(partitioning)
            if part_key is None:
                raise ValueError(f"不支持的分区粒度: {partitioning}，可选: 年、月、日")
        else:
            part_key = None

        # 更新配置（含字段描述）
        merged_schema = self._infer_schema(data)
        if schema:
            for col, info in schema.items():
                if col in merged_schema:
                    merged_schema[col].update(info)
                else:
                    merged_schema[col] = info
        self._ensure_config(id, date_col, part_key, unique_together, merged_schema)

        df = data.copy()
        df[date_col] = pd.to_datetime(df[date_col])

        table_dir = self.base_dir / id

        if part_key is None:
            # 不分区：全部写入单个文件
            table_dir.mkdir(parents=True, exist_ok=True)
            parquet_path = table_dir / "data.parquet"
            self._write_parquet(parquet_path, df, unique_together, rewrite)
        else:
            # 按分区分组写入
            df["_pk"] = df[date_col].apply(lambda d: self._partition_key(d, part_key))
            for pk, group in df.groupby("_pk"):
                group = group.drop(columns=["_pk"])
                sub_dir = table_dir / self._partition_dir(part_key, pk)
                sub_dir.mkdir(parents=True, exist_ok=True)
                parquet_path = sub_dir / "data.parquet"
                self._write_parquet(parquet_path, group, unique_together, rewrite)

        # 刷新该表的视图
        self._create_view(id)

    def _write_parquet(self, parquet_path, df_new, unique_together, rewrite):
        """写入单个 parquet 文件，处理合并去重。"""
        if rewrite or not parquet_path.exists():
            # rewrite=True：全量覆盖，直接用新数据，不读旧数据
            df_merged = df_new
        else:
            # rewrite=False：增量合并，新数据优先覆盖旧数据中的冲突行
            df_old = pd.read_parquet(parquet_path)
            if unique_together:
                df_merged = self._merge_dedup(df_old, df_new, unique_together)
            else:
                df_merged = pd.concat([df_old, df_new], ignore_index=True)

        if unique_together:
            df_merged = df_merged.sort_values(unique_together).reset_index(drop=True)
        else:
            df_merged = df_merged.reset_index(drop=True)

        df_merged.to_parquet(parquet_path, index=False, compression="zstd")
        print(f"  [写入] {parquet_path}  ({len(df_merged)} 行)")

    def query(self, sql: str):
        """
        执行 SQL 查询，返回 DuckDB 的结果对象。
        调用 .df() 可转为 pandas DataFrame。
        """
        return self._con.execute(sql)

    def tables(self) -> list:
        """返回已注册的所有表名列表。"""
        return list(self._config.keys())

    def delete(self, id: str):
        """删除指定表：移除数据文件、DuckDB 视图和配置。"""
        folder = self.base_dir / id
        if folder.exists():
            shutil.rmtree(folder)

        try:
            self._con.execute(f"DROP VIEW IF EXISTS {id}")
        except Exception:
            pass

        if id in self._config:
            del self._config[id]
            self._save_config()

        print(f"  [删除] 表 '{id}' 已移除")

    def set_schema(self, id: str, schema: dict):
        """
        为已有表设置或更新字段描述，无需重新写入数据。

        参数:
            id:      表名
            schema:  字段描述，如 {"close": {"desc": "收盘价", "type": "float"}}
                     已有字段的描述会被合并更新，新字段会追加。
        """
        if id not in self._config:
            raise ValueError(f"表 '{id}' 不存在")
        existing = self._config[id].get("schema", {})
        for col, info in schema.items():
            if col in existing:
                existing[col].update(info)
            else:
                existing[col] = info
        self._config[id]["schema"] = existing
        self._save_config()

    def describe(self, id: str) -> pd.DataFrame:
        """
        返回指定表的字段描述信息。

        返回 DataFrame 包含: 字段、物理类型、说明、是否主键。
        """
        cfg = self._config.get(id)
        if cfg is None:
            raise ValueError(f"表 '{id}' 不存在")

        folder = self.base_dir / id
        sample = next(folder.rglob("*.parquet"), None)
        if sample is None:
            return pd.DataFrame(columns=["字段", "物理类型", "说明", "是否主键"])

        sample_path = str(sample).replace("\\", "/")
        cols = self._con.execute(
            f"SELECT name, type FROM parquet_schema('{sample_path}') "
            "WHERE num_children IS NULL"
        ).fetchall()

        schema = cfg.get("schema", {})
        unique_keys = cfg.get("unique_keys", [])
        rows = []
        for name, dtype in cols:
            info = schema.get(name, {})
            rows.append({
                "字段": name,
                "物理类型": dtype,
                "说明": info.get("desc", ""),
                "是否主键": name in unique_keys,
            })
        return pd.DataFrame(rows)

    def ask(
        self,
        question: str,
        *,
        return_df: bool = True,
        model: Optional[str] = None,
    ):
        """
        用自然语言查询数据库，AI 自动生成 SQL 并执行。

        参数:
            question:   自然语言问题，如 "2024年收盘价最高的前10只股票"
            return_df:  True 返回查询结果 DataFrame，False 返回生成的 SQL 字符串
            model:      模型名称，不传则使用构造函数中指定的模型
        """
        try:
            from openai import OpenAI
        except ImportError:
            raise ImportError(
                "使用 ask() 需要安装 openai，请运行: pip install xxydb[ai]"
            )

        api_key = self._api_key or os.environ.get("OPENAI_API_KEY")
        if not api_key:
            raise ValueError(
                "未提供 API Key，请通过构造函数参数 api_key "
                "或环境变量 OPENAI_API_KEY 设置"
            )

        if not self._config:
            raise ValueError("数据库中没有任何表，请先写入数据")

        model = model or self._model
        if not model:
            raise ValueError(
                "未指定模型，请通过构造函数参数 model 或 ask(model=...) 设置"
            )

        system_prompt = self._build_schema_prompt()
        client_kwargs = {"api_key": api_key}
        base_url = self._base_url or os.environ.get("OPENAI_BASE_URL")
        if base_url:
            client_kwargs["base_url"] = base_url
        client = OpenAI(**client_kwargs)
        resp = client.chat.completions.create(
            model=model,
            messages=[
                {"role": "system", "content": system_prompt},
                {"role": "user", "content": question},
            ]
        )

        raw = resp.choices[0].message.content.strip()
        sql = self._extract_sql(raw)
        self._validate_sql(sql)

        if not return_df:
            return sql

        return self._con.execute(sql).df()

    def close(self):
        """关闭 DuckDB 连接。"""
        self._con.close()

    # ──────────────────────────────────────────
    # 内部方法
    # ──────────────────────────────────────────

    def _build_schema_prompt(self) -> str:
        """将所有表的 schema 元数据组装为系统提示词。"""
        lines = [
            "你是一个 DuckDB SQL 生成助手。根据用户的自然语言问题，生成准确的 SQL 查询。",
            "",
            "可用的表及其字段：",
        ]
        for table_id, cfg in self._config.items():
            lines.append(f"\n## {table_id}")
            schema = cfg.get("schema", {})
            unique_keys = cfg.get("unique_keys", [])
            for col, info in schema.items():
                dtype = info.get("type", "")
                desc = info.get("desc", "")
                pk = " [主键]" if col in unique_keys else ""
                parts = [f"  - {col}"]
                if dtype:
                    parts.append(f"({dtype})")
                if desc:
                    parts.append(f": {desc}")
                if pk:
                    parts.append(pk)
                lines.append(" ".join(parts))
        lines.extend([
            "",
            "规则：",
            "1. 只生成 SELECT 语句（可使用 WITH/CTE），禁止 INSERT/UPDATE/DELETE/DROP 等操作",
            "2. 只返回纯 SQL，不要添加解释、注释或 markdown 格式",
            "3. 使用 DuckDB SQL 语法",
            "4. 日期比较使用标准格式如 '2024-01-01'",
        ])
        return "\n".join(lines)

    @staticmethod
    def _extract_sql(text: str) -> str:
        """从 AI 返回的文本中提取 SQL 语句（兼容 markdown 代码块）。"""
        m = re.search(r"```(?:sql)?\s*\n?(.*?)\n?```", text, re.DOTALL)
        if m:
            return m.group(1).strip()
        return text.strip()

    @staticmethod
    def _validate_sql(sql: str):
        """校验 SQL 安全性：只允许 SELECT / WITH 开头，禁止修改类操作。"""
        normalized = sql.strip().upper()
        if not (normalized.startswith("SELECT") or normalized.startswith("WITH")):
            raise ValueError(f"AI 生成了非查询语句，已拒绝执行: {sql[:100]}")
        forbidden = [
            "INSERT", "UPDATE", "DELETE", "DROP",
            "ALTER", "CREATE", "TRUNCATE", "REPLACE",
        ]
        for kw in forbidden:
            if re.search(rf"\b{kw}\b", normalized):
                raise ValueError(f"SQL 包含禁止的操作 '{kw}'，已拒绝执行")

    @staticmethod
    def _infer_schema(df: pd.DataFrame) -> dict:
        """从 DataFrame 自动推断字段类型信息，desc 留空待人工补充。"""
        schema = {}
        for col in df.columns:
            schema[col] = {"type": str(df[col].dtype), "desc": ""}
        return schema

    @staticmethod
    def _partition_key(dt: pd.Timestamp, part_key: str):
        if part_key == "day":
            return (dt.year, dt.month, dt.day)
        if part_key == "month":
            return (dt.year, dt.month)
        return (dt.year,)

    @staticmethod
    def _partition_dir(part_key: str, pk: tuple) -> str:
        if part_key == "day":
            return f"year={pk[0]}/month={pk[1]:02d}/day={pk[2]:02d}"
        if part_key == "month":
            return f"year={pk[0]}/month={pk[1]:02d}"
        return f"year={pk[0]}"

    @staticmethod
    def _merge_dedup(
        df_old: pd.DataFrame,
        df_new: pd.DataFrame,
        keys: List[str],
    ) -> pd.DataFrame:
        """合并去重。新数据在后，keep='last' 保留新数据（仅在 rewrite=False 时调用）。"""
        merged = pd.concat([df_old, df_new], ignore_index=True)
        return merged.drop_duplicates(subset=keys, keep="last")

    def _load_config(self) -> dict:
        if self._config_path.exists():
            with open(self._config_path, "r", encoding="utf-8") as f:
                return json.load(f)
        return {}

    def _save_config(self):
        with open(self._config_path, "w", encoding="utf-8") as f:
            json.dump(self._config, f, ensure_ascii=False, indent=4)

    def _ensure_config(
        self, table_id: str, date_col: str, part_key: str, unique_together,
        schema: Optional[dict] = None,
    ):
        """如果表不在配置中则自动添加；已有表则合并更新 schema。"""
        if table_id not in self._config:
            self._config[table_id] = {
                "partition_by": part_key,
                "date_column": date_col,
                "unique_keys": unique_together or [],
                "source_folder": table_id,
                "schema": schema or {},
            }
        else:
            # 合并新 schema 到已有配置（新字段追加，已有字段补充）
            existing = self._config[table_id].get("schema", {})
            if schema:
                for col, info in schema.items():
                    if col in existing:
                        existing[col].update(info)
                    else:
                        existing[col] = info
            self._config[table_id]["schema"] = existing
        self._save_config()

    def _create_view(self, table_id: str):
        """为指定表创建/刷新 DuckDB 视图。"""
        folder = self.base_dir / table_id
        glob_pattern = str(folder / "**" / "*.parquet").replace("\\", "/")

        sample = next(folder.rglob("*.parquet"), None)
        if sample is None:
            return

        cfg = self._config.get(table_id, {})
        part_key = cfg.get("partition_by")

        if part_key:
            # 有分区：开启 hive_partitioning，但只 SELECT 原始列
            sample_path = str(sample).replace("\\", "/")
            cols = self._con.execute(
                f"SELECT name FROM parquet_schema('{sample_path}') WHERE num_children IS NULL"
            ).fetchall()
            col_names = ", ".join(f'"{c[0]}"' for c in cols)

            sql = f"""
                CREATE OR REPLACE VIEW {table_id} AS
                SELECT {col_names} FROM read_parquet(
                    '{glob_pattern}',
                    hive_partitioning = true,
                    union_by_name = true
                );
            """
        else:
            # 无分区：直接读取
            sql = f"""
                CREATE OR REPLACE VIEW {table_id} AS
                SELECT * FROM read_parquet(
                    '{glob_pattern}',
                    union_by_name = true
                );
            """
        self._con.execute(sql)

    def _refresh_all_views(self):
        """启动时为所有已配置的表刷新视图。"""
        for table_id in self._config:
            folder = self.base_dir / table_id
            if folder.exists():
                self._create_view(table_id)



import shutil
from pathlib import Path

import pandas as pd
import pytest

from xxydb import xxydb


@pytest.fixture
def tmp_db(tmp_path):
    """创建一个临时目录下的 xxydb 实例，测试结束后自动清理。"""
    db = xxydb(path=str(tmp_path / "testdb"))
    yield db
    db.close()


@pytest.fixture
def sample_df():
    """生成跨两年的示例 DataFrame。"""
    return pd.DataFrame(
        {
            "date": pd.to_datetime(
                ["2023-06-01", "2023-12-15", "2024-03-10", "2024-07-20"]
            ),
            "code": ["000001", "000002", "000001", "000002"],
            "close": [10.0, 20.0, 30.0, 40.0],
        }
    )


# ──────────────────────────────────────────
# 基本功能
# ──────────────────────────────────────────


class TestInit:
    def test_empty_database(self, tmp_db):
        assert tmp_db.tables() == []

    def test_base_dir_created(self, tmp_db):
        assert tmp_db.base_dir.exists()


class TestWritePartitioned:
    def test_write_and_query(self, tmp_db, sample_df):
        tmp_db.write_data(
            sample_df,
            id="bar",
            date_col="date",
            partitioning="年",
            unique_together=["date", "code"],
        )
        result = tmp_db.query("SELECT * FROM bar").df()
        assert len(result) == 4
        assert set(result.columns) == {"date", "code", "close"}

    def test_partition_dirs_created(self, tmp_db, sample_df):
        tmp_db.write_data(sample_df, id="bar", date_col="date", partitioning="年")
        table_dir = tmp_db.base_dir / "bar"
        years = sorted(p.name for p in table_dir.iterdir() if p.is_dir())
        assert years == ["year=2023", "year=2024"]

    def test_month_partition(self, tmp_db, sample_df):
        tmp_db.write_data(sample_df, id="bar", date_col="date", partitioning="月")
        table_dir = tmp_db.base_dir / "bar"
        parquet_files = list(table_dir.rglob("*.parquet"))
        # 4 rows span 4 distinct year-month combos
        assert len(parquet_files) == 4


class TestWriteUnpartitioned:
    def test_write_no_partition(self, tmp_db, sample_df):
        tmp_db.write_data(
            sample_df, id="flat", date_col="date", partitioning=None
        )
        result = tmp_db.query("SELECT * FROM flat").df()
        assert len(result) == 4

    def test_single_parquet_file(self, tmp_db, sample_df):
        tmp_db.write_data(
            sample_df, id="flat", date_col="date", partitioning=None
        )
        parquet_files = list((tmp_db.base_dir / "flat").rglob("*.parquet"))
        assert len(parquet_files) == 1


class TestDedup:
    def test_rewrite_true_keeps_new(self, tmp_db):
        df1 = pd.DataFrame(
            {"date": ["2024-01-01"], "code": ["000001"], "close": [10.0]}
        )
        df2 = pd.DataFrame(
            {"date": ["2024-01-01"], "code": ["000001"], "close": [99.0]}
        )
        tmp_db.write_data(
            df1, id="t", date_col="date", partitioning="年",
            unique_together=["date", "code"],
        )
        tmp_db.write_data(
            df2, id="t", date_col="date", partitioning="年",
            unique_together=["date", "code"], rewrite=True,
        )
        result = tmp_db.query("SELECT close FROM t").df()
        assert result["close"].iloc[0] == pytest.approx(99.0)

    def test_rewrite_false_keeps_old(self, tmp_db):
        df1 = pd.DataFrame(
            {"date": ["2024-01-01"], "code": ["000001"], "close": [10.0]}
        )
        df2 = pd.DataFrame(
            {"date": ["2024-01-01"], "code": ["000001"], "close": [99.0]}
        )
        tmp_db.write_data(
            df1, id="t", date_col="date", partitioning="年",
            unique_together=["date", "code"],
        )
        tmp_db.write_data(
            df2, id="t", date_col="date", partitioning="年",
            unique_together=["date", "code"], rewrite=False,
        )
        result = tmp_db.query("SELECT close FROM t").df()
        assert result["close"].iloc[0] == pytest.approx(10.0)


class TestDelete:
    def test_delete_removes_table(self, tmp_db, sample_df):
        tmp_db.write_data(sample_df, id="todel", date_col="date", partitioning="年")
        assert "todel" in tmp_db.tables()
        tmp_db.delete("todel")
        assert "todel" not in tmp_db.tables()
        assert not (tmp_db.base_dir / "todel").exists()

    def test_delete_nonexistent_no_error(self, tmp_db):
        tmp_db.delete("nonexistent")  # should not raise


class TestTables:
    def test_tables_lists_written(self, tmp_db, sample_df):
        tmp_db.write_data(sample_df, id="a", date_col="date", partitioning=None)
        tmp_db.write_data(sample_df, id="b", date_col="date", partitioning="年")
        assert sorted(tmp_db.tables()) == ["a", "b"]


# ──────────────────────────────────────────
# 上下文管理器
# ──────────────────────────────────────────


class TestContextManager:
    def test_with_statement(self, tmp_path):
        with xxydb(path=str(tmp_path / "ctx")) as db:
            assert db.tables() == []
            db.write_data(
                pd.DataFrame({"date": ["2024-01-01"], "v": [1]}),
                id="x",
                date_col="date",
                partitioning=None,
            )
            assert db.tables() == ["x"]
        # 连接已关闭，再次操作应报错
        with pytest.raises(Exception):
            db.query("SELECT 1")

    def test_repr(self, tmp_db, sample_df):
        tmp_db.write_data(sample_df, id="bar", date_col="date", partitioning="年")
        r = repr(tmp_db)
        assert "xxydb" in r
        assert "tables=1" in r


# ──────────────────────────────────────────
# 异常
# ──────────────────────────────────────────


class TestSchema:
    def test_auto_infer_schema(self, tmp_db, sample_df):
        """写入时不传 schema，自动推断 dtype 并记录。"""
        tmp_db.write_data(sample_df, id="bar", date_col="date", partitioning="年",
                          unique_together=["date", "code"])
        cfg = tmp_db._config["bar"]
        assert "schema" in cfg
        assert "date" in cfg["schema"]
        assert "code" in cfg["schema"]
        assert "close" in cfg["schema"]
        # 自动推断时 desc 为空
        assert cfg["schema"]["close"]["desc"] == ""
        # type 应来自 pandas dtype
        assert cfg["schema"]["close"]["type"] != ""

    def test_write_with_schema(self, tmp_db, sample_df):
        """写入时传入 schema，desc 应被正确保存。"""
        schema = {
            "date": {"desc": "交易日期"},
            "code": {"desc": "股票代码（6位）"},
            "close": {"desc": "日收盘价（元）"},
        }
        tmp_db.write_data(sample_df, id="bar", date_col="date", partitioning="年",
                          unique_together=["date", "code"], schema=schema)
        cfg = tmp_db._config["bar"]["schema"]
        assert cfg["date"]["desc"] == "交易日期"
        assert cfg["code"]["desc"] == "股票代码（6位）"
        assert cfg["close"]["desc"] == "日收盘价（元）"
        # 自动推断的 type 也应保留
        assert "type" in cfg["close"]

    def test_set_schema_updates_existing(self, tmp_db, sample_df):
        """set_schema 可以为已有表补充或更新字段描述。"""
        tmp_db.write_data(sample_df, id="bar", date_col="date", partitioning="年")
        # 初始无描述
        assert tmp_db._config["bar"]["schema"]["close"]["desc"] == ""
        # 补充描述
        tmp_db.set_schema("bar", {"close": {"desc": "日收盘价（元）"}})
        assert tmp_db._config["bar"]["schema"]["close"]["desc"] == "日收盘价（元）"
        # 原有 type 不丢失
        assert "type" in tmp_db._config["bar"]["schema"]["close"]

    def test_set_schema_nonexistent_table(self, tmp_db):
        """对不存在的表调用 set_schema 应报错。"""
        with pytest.raises(ValueError, match="不存在"):
            tmp_db.set_schema("ghost", {"x": {"desc": "test"}})

    def test_describe_returns_dataframe(self, tmp_db, sample_df):
        """describe 返回包含字段描述的 DataFrame。"""
        schema = {
            "date": {"desc": "交易日期"},
            "code": {"desc": "股票代码"},
            "close": {"desc": "收盘价"},
        }
        tmp_db.write_data(sample_df, id="bar", date_col="date", partitioning="年",
                          unique_together=["date", "code"], schema=schema)
        desc = tmp_db.describe("bar")
        assert list(desc.columns) == ["字段", "物理类型", "说明", "是否主键"]
        assert len(desc) == 3
        # 检查主键标记
        code_row = desc[desc["字段"] == "code"].iloc[0]
        assert code_row["是否主键"] is True
        assert code_row["说明"] == "股票代码"
        close_row = desc[desc["字段"] == "close"].iloc[0]
        assert close_row["是否主键"] is False

    def test_describe_nonexistent_table(self, tmp_db):
        """对不存在的表调用 describe 应报错。"""
        with pytest.raises(ValueError, match="不存在"):
            tmp_db.describe("ghost")

    def test_schema_merge_on_second_write(self, tmp_db):
        """第二次写入时 schema 应合并，不覆盖已有描述。"""
        df1 = pd.DataFrame({"date": ["2024-01-01"], "code": ["000001"], "close": [10.0]})
        schema1 = {"close": {"desc": "收盘价"}}
        tmp_db.write_data(df1, id="t", date_col="date", partitioning="年", schema=schema1)
        assert tmp_db._config["t"]["schema"]["close"]["desc"] == "收盘价"

        # 第二次写入，不传 close 的 schema，但传 code 的
        df2 = pd.DataFrame({"date": ["2024-02-01"], "code": ["000002"], "close": [20.0]})
        schema2 = {"code": {"desc": "股票代码"}}
        tmp_db.write_data(df2, id="t", date_col="date", partitioning="年", schema=schema2)
        # close 的描述应保留
        assert tmp_db._config["t"]["schema"]["close"]["desc"] == "收盘价"
        # code 的描述应更新
        assert tmp_db._config["t"]["schema"]["code"]["desc"] == "股票代码"


class TestErrors:
    def test_missing_date_col(self, tmp_db):
        df = pd.DataFrame({"not_date": ["2024-01-01"], "v": [1]})
        with pytest.raises(ValueError, match="日期列"):
            tmp_db.write_data(df, id="err", date_col="date")

    def test_invalid_partitioning(self, tmp_db):
        df = pd.DataFrame({"date": ["2024-01-01"], "v": [1]})
        with pytest.raises(ValueError, match="不支持的分区粒度"):
            tmp_db.write_data(df, id="err", date_col="date", partitioning="周")
